#!/usr/bin/perl -w
use strict;
use POSIX;
use FileHandle;
use Data::Dumper;

my %epg = ();

# recfriio
my $cmd_recfriio = "./recfriio --b25 --strip";
# epgdump
my $cmd_epgdump  = "./epgdump";

die "usage: $0 epg.xmltv" if @ARGV != 1;
my $file = $ARGV[0];
my $fh = FileHandle->new("< $file") or die "can't open $file to read: $!";
while (my $line = $fh->getline) {
        if ($line =~ m/^channel\s+(\d+)\.(\d+)\.(uhf|bs|cs)\.isdb\.epg$/) {
                my ($chId, $pyChannel, $pyBand) = ($1, $2, $3);
                $epg{"$pyChannel.$pyBand.isdb.epg"}{$chId} = {};
        }
}
$fh->close();

for my $target (sort { $a cmp $b } keys %epg) {
	if ($target =~ m/^(\d+)\.(uhf|bs|cs)\.isdb\.epg$/) {
		my ($pyChannel, $pyBand) = ($1, $2);
		
		my $pyName = "$pyChannel.$pyBand.isdb.epg";	
	
		my %bandHash = ( "uhf" => "", "bs" => "B", "cs" => "C" );
		$pyChannel = "$bandHash{$pyBand}$pyChannel" if exists $bandHash{$pyBand};
		my $reccmd = "$cmd_recfriio $pyChannel - - 2>/dev/null | $cmd_epgdump -l";
		my $epgin = FileHandle->new("$reccmd |") or die "failed to get epg : $!";
		my $fromtime = time();
		my $count = 0;
		my $fillstat = 0;
		while (my $line = $epgin->getline) {
			print("Program table filled.\n"), last if $fillstat && $line =~ m/^EIT,/;
			
			parse($epg{$pyName}, $line);
			
			if (++$count % 500 == 0) {
				printf "time: %d count: %d\n", time() - $fromtime, $count;
				my $stat = 1;
				for my $sv (sort { $a <=> $b } keys %{$epg{$pyName}}) {
					$stat = status($epg{$pyName}{$sv}, $sv) && $stat;
				}
				$fillstat = $stat if time() - $fromtime > 60;
				flush STDOUT;
			}
		}
	}
}
dumpepg();

exit 0;

########################

sub parse {
	my ($pych, $line) = @_;
	chomp $line;

#SVC,chId,chName,?
	if ($line =~ m/^SVC,([0-9]+),(.*),(.*)$/) {
		my ($serviceId, $name, $unknown) = (int($1), $2, $3);
		return if !exists $pych->{$serviceId};

		$pych->{$serviceId}{name} = $name;
		return;
	}

#EIT,table_id=4e,service_id=23615,ver=12,next=1,section=01,lastsection=01,seglastsection=01,lasttable=4e
	if ($line =~ m/^EIT,table_id=([0-9a-f]+),service_id=([0-9]+),ver=([0-9]+),next=([01]),section=([0-9a-f]+),lastsection=([0-9a-f]+),seglastsection=([0-9a-f]+),lasttable=([0-9a-f]+)$/) {
		my ($tableId, $serviceId, $version, $next, $section, $lastSection, $segLastSection, $lastTable)
			= (hex($1), int($2), int($3), int($4), hex($5), hex($6), hex($7), hex($8));
		
		if ($next ne "1") {
			print "next != 1 : $line\n";
			return;
		}
		return if !exists $pych->{$serviceId};
		#return if $tableId < 0x50 || 0x5f < $tableId;
		return if $tableId < 0x50 || 0x6f < $tableId;

		$pych->{$serviceId}{eit} = {} if !exists $pych->{$serviceId}{eit};
		my $sv = $pych->{$serviceId}{eit};

		$sv->{$tableId}{count} = 0 if !exists $sv->{$tableId};
		my $table = $sv->{$tableId};
		$table->{count}++;
		$table->{to} = $lastTable;
		$table->{lastsec} = $lastSection;

		$table->{version} = $version if !defined $table->{version};
		if ($table->{version} != $version) {
			print "version changed: $version $line\n";
			delete $pych->{$serviceId}{eit};
			delete $pych->{$serviceId}{epg};
			$pych->{$serviceId}{eit}{$tableId}{count} = 0;
			return;
		}

		$table->{secs}{$section}{count} = 0 if !exists $table->{secs}{$section}{count};
		my $sec = $table->{secs}{$section};
		$sec->{count}++;
		$sec->{to} = $lastSection;
		$sec->{segto} = $segLastSection;

		return;
	}

#SEVT,23608 chId,46126 pgmId,-1,2008/11/08 day,03:30:00 from,04:00:00 to,00:30:00 len,1800 len,title,desc
	if ($line =~ m/^SEVT,([0-9]+),([0-9]+),-1,([\/0-9]+),([:0-9]+),([:0-9]+),([:0-9]+),([0-9]+),(.*),(.*)$/) {
		my ($serviceId, $progId, $day, $from, $to, $len, $sec, $title, $desc)
			= (int($1), int($2), $3, $4, $5, $6, $7, $8, $9);

		return if !exists $pych->{$serviceId};
		$pych->{$serviceId}{epg} = {} if !exists $pych->{$serviceId}{epg};
		my $sv = $pych->{$serviceId}{epg};

		$sv->{$progId}{from}  = "$day $from";
		$sv->{$progId}{to}    = "$day $to";
		$sv->{$progId}{len}   = $len;
		$sv->{$progId}{sec}   = $sec;
		$sv->{$progId}{title} = $title;
		$sv->{$progId}{desc}  = $desc;

		return;
	}
	
#EEVT,chId,pgmId,order,descType,desc
	if ($line =~ m/^EEVT,([0-9]+),([0-9]+),([0-9]+),(.*),(.*)$/) {
		my ($serviceId, $progId, $order, $type, $desc) = (int($1), int($2), $3, $4, $5);
		
		return if !exists $pych->{$serviceId};
		$pych->{$serviceId}{epg} = {} if !exists $pych->{$serviceId}{epg};
		my $sv = $pych->{$serviceId}{epg};

		$sv->{$progId}{ext}{$order}{type} = $type;
		$sv->{$progId}{ext}{$order}{desc} = $desc;

		return;
	}
	
	return if $line =~ m/^EEVTt/;
	#return if $line =~ m/^Content/;
	if ($line =~ m/^Content,([0-9]+),([0-9]+),(.*)$/) {
		my ($serviceId, $progId, $content) = (int($1), int($2), $3);

		return if !exists $pych->{$serviceId};
		$pych->{$serviceId}{epg} = {} if !exists $pych->{$serviceId}{epg};
		my $sv = $pych->{$serviceId}{epg};

		$sv->{$progId}{content} = $content;

		return;
	}

	print "unknown: $line\n";
}

sub status {
	my ($sv, $id) = @_;
	if (!exists $sv->{eit}) {
		#print "sv: $id:$sv->{name} empty.\n";
		return 1;
	}

	print "sv: $id:$sv->{name}\n";
	my $exed  = 0;
	my $extab = 0;
	my $tab   = 0;
	my $filledtab = 0;
	my $prefilledtab = 0;
	my $prefilledtab2 = 0;
	my $mineitcount = undef;
	for my $t (sort { $a <=> $b } keys %{$sv->{eit}}) {
		$tab++;
		if ($t > $exed) {
			$exed   = $sv->{eit}{$t}{to};
			$extab += $sv->{eit}{$t}{to} - $t + 1;
		}
	}
	print "tables: $tab/$extab\n";

	my @lt = localtime;
	my $segoffset = int($lt[2] / 3);

	my $tabgrp = 0;
	for my $t (sort { $a <=> $b } keys %{$sv->{eit}}) {
		printf "%02x: %d", $t, $sv->{eit}{$t}{count};
		
		my $grphead = 0;
		$grphead = 1 if $sv->{eit}{$t}{to} != $tabgrp;
		$tabgrp = $sv->{eit}{$t}{to};

		my $exseg  = int(ceil(($sv->{eit}{$t}{lastsec}+1)/8)) - ($grphead ? $segoffset : 0);
		my $segcnt = 0;
		my $presegcnt = 0;
		for (my $segst = ($grphead ? $segoffset * 8 : 0); $segst <= 0xff; $segst += 8) {
			next if !exists $sv->{eit}{$t}{secs}{$segst};
			my $segto = undef;
			my $realseglen = 0;
			for (my $s = $segst; $s < $segst + 8; $s++) {
				next if !exists $sv->{eit}{$t}{secs}{$s};
				$segto = $sv->{eit}{$t}{secs}{$s}{segto};
				$mineitcount = $sv->{eit}{$t}{secs}{$s}{count} if !defined $mineitcount || $sv->{eit}{$t}{secs}{$s}{count} < $mineitcount;
				$realseglen++;
			}
			if (defined $segto) {
				my $seglen = $segto - $segst + 1;
				$segcnt++    if $seglen == $realseglen;
			}
		}
		print "\tsegment: $segcnt/$exseg\n";
		$filledtab++     if $segcnt == $exseg;
		#for my $s (sort { $a <=> $b } keys %{$sv->{eit}{$t}{secs}}) {
		#	my $sec = $sv->{eit}{$t}{secs}{$s};
		#	printf "sec: %02x seglast: %02x lastsec: %02x count: %d\n",
		#		$s, $sec->{segto}, $sec->{to}, $sec->{count};
		#}
	}
	print "\n";
	
	return $extab == $filledtab;
}

sub dumpepg {
	for my $py (sort { $a cmp $b } keys %epg) {
		print "Channel(py): $py\n";
		my $pyv = $epg{$py};
		for my $sv (sort { $a <=> $b } keys %$pyv) {
			print "Channel: $sv($pyv->{$sv}{name})\n";
			my $svepg = $pyv->{$sv}{epg};
			my @programs = map { $_->[1] } sort { $a->[0] cmp $b->[0] } map { [ $_->{from}, $_ ] } values %$svepg;
			print "Progs:\n";
			for my $p (@programs) {
				print "$p->{from} $p->{to} $p->{title}\n";
				print "\t'$p->{desc}'\n" if defined $p->{desc} && "" ne $p->{desc};
				print "\t$p->{content}\n" if defined $p->{content};
				for my $exorder (sort { $a <=> $b } keys %{$p->{ext}}) {
					my $ex = $p->{ext}{$exorder};
					print "\t$exorder\t$ex->{type}\t$ex->{desc}\n";
				}
			}
		}
	}
}

